#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

int main(int argv, char *argc[]) {
  FILE *in;
  int linesize=100; // initial maximum line length.  Will increase if necessary 
  char *line = malloc(linesize*sizeof(char));
  fpos_t position;
  int nread = 0;
  int xsize = 10;
  double *x = malloc(xsize*sizeof(double));
  const char delim[]=","; 
  int i,j;

  if (line==NULL) {
    printf("Failed to allocate %d bytes\n",linesize*sizeof(char));
    exit(1);
  }
  if (argv!=2) {
    printf("Requires filename as command line input.");
    exit(1);
  }
  if (!(in = fopen(argc[1],"r"))) {
    printf("Failed to open %s\n", argc[1]);
    exit(1);
  } 
  fgetpos(in,&position);
  while(fgets(line,linesize,in)!=NULL) {
    // find newline character
    char *newline = strchr(line,'\n'); 
    if (newline==NULL) { // linesize too small
      printf("Encountered a line with more than %d " 
	     "characters, increasing linesize to %d.\n",
	     linesize,linesize*2);
      linesize*=2;
      free(line);
      line = malloc(linesize*sizeof(char));
      if (line==NULL) {
	printf("Failed to allocate %d bytes\n",linesize*sizeof(char));
	exit(1);
      }
      fsetpos(in,&position); // go back and try reading this line again
    } else { // successfully stored entire line in line
      char *field;
      fgetpos(in,&position); // update position
      // break line into fields
      field = strtok(line,delim);
      while(field!=NULL) {
	if (nread>=xsize) {
	  xsize *= 2;
	  printf("Reallocating x to hold %d doubles.\n",xsize);
	  x = realloc(x,xsize*sizeof(double));
	  if (x==NULL) {
	    printf("Failed to reallocated space for x\n");
	    exit(1);
	  }
	}
	if (0==sscanf(field,"%lf",x+nread)) {
	  x[nread] = NAN; 
	}
	nread++;
	field=strtok(NULL,delim);
      } // end while(field!=NULL)
    }
  } // end while
  free(line);
  fclose(in);
  // print the numbersx
  printf("Read %d entries from %s.\n",nread,argc[1]);
  i = 0;
  while(i<nread) {
    for(j=0;j<6 && i<nread;j++) printf("%10.6g ",x[i++]);
    printf("\n");
  }
  free(x);
  return(0);
}
