/* 
   Paul Schrimpf
   May 30, 2008
   Plugin for stata that generates fibonacci numbers
   See http://www.stata.com/plugins/ for more info  
*/

#include "stplugin.h"
#include <stdio.h>
#ifdef OMP
#include <omp.h>
#endif

unsigned int fibLoop(unsigned int n);
STDLL stata_call(int argc, char *argv[]) 
{
  /* these must be special data types defined in stplugin.h */
  ST_int      nObs = SF_in2() - SF_in1();
  ST_double   val,fib;
  ST_retcode  rc ;
  ST_int start = SF_in1();
  ST_int end = SF_in2();
  ST_int j;
  
  if(SF_nvars() != 2) { 
    return(102) ;      /* wrong number of variables specified */
  }

#pragma omp parallel default(shared)   
  {
#ifdef OMP
    int tid = omp_get_thread_num();
    int threads = omp_get_num_threads();
    ST_int work =  (ST_int) ((double) nObs)/((double) threads);
    char message[100];
    sprintf(message,"%d threads\n",omp_get_num_threads());
    if (tid==0) SF_display(message);
    sprintf(message,"thread %d says hello, working on %d obs\n",tid,work);
    SF_display(message);
    /*start = SF_in1() + tid*work;
      end = SF_in1() + (tid+1)*work;
      if (tid==threads-1) end = SF_in2(); */
#endif
#pragma omp for
    for(j = start; j <= end; j++) { 
      /* loop from first to last observation in range */ 
      if(SF_ifobs(j)) { /* true if stata if command true */
        /* retrieve j-th value of 1-st variable passed to command */
        SF_vdata(1,j,&val);
        /* calculate fib */
        fib = SF_is_missing(val)? SV_missval :
          (ST_double) fibLoop((unsigned int) val);
        /* store result in j-th value of last variable passed to command */
        SF_vstore(SF_nvars(), j, fib);
      }
    }
  } /* end parallel region */
  return(rc=0) ;  
}


/*
  Returns the nth Fibonacci number.
  Will overflow for n too large.
*/
unsigned int fibLoop(unsigned int n) {
  unsigned int F,Fm1,Fm2;
  int i;
  
  if (n<=1) return(n);
  Fm1=1;
  Fm2=0;
  for(i=2; i<=n; i++) {
    F = Fm2+Fm1;
    Fm2 = Fm1;
    Fm1 = F;
  }
  return(F);
}

