/* 
   Paul Schrimpf
   May 30, 2008
   Plugin for stata that generates fibonacci numbers
   See http://www.stata.com/plugins/ for more info  
*/

#include "stplugin.h"

unsigned int fibLoop(unsigned int n);
STDLL stata_call(int argc, char *argv[]) 
{
  /* these must be special data types defined in stplugin.h */
  ST_int      j;
  ST_double   val,fib;
  ST_retcode  rc ;
  
  if(SF_nvars() != 2) { 
    return(102) ;      /* wrong number of variables specified */
  }
  
  for(j = SF_in1(); j <= SF_in2(); j++) { 
    /* loop from first to last observation in range */ 
    if(SF_ifobs(j)) { /* true if stata if command true */
      /* retrieve j-th value of 1-st variable passed to command */
      if(rc = SF_vdata(1,j,&val)) return(rc) ;
      /* calculate fib */
      fib = SF_is_missing(val)? SV_missval :
        (ST_double) fibLoop((unsigned int) val);
      /* store result in j-th value of last variable passed to command */
      if(rc = SF_vstore(SF_nvars(), j, fib)) return(rc) ;
    }
  }
  return(0) ;  
}


/*
  Returns the nth Fibonacci number.
  Will overflow for n too large.
*/
unsigned int fibLoop(unsigned int n) {
  unsigned int F,Fm1,Fm2;
  int i;
  
  if (n<=1) return(n);
  Fm1=1;
  Fm2=0;
  for(i=2; i<=n; i++) {
    F = Fm2+Fm1;
    Fm2 = Fm1;
    Fm1 = F;
  }
  return(F);
}

